#pragma once

#if defined(WIN32)
#include "ITinyWindow.h"
#include "HttpSession_win32.h"
#else
#include "IGTEventDelegate.h"
#include "HttpSession_bsdsocket.h"
#endif
#include <list>
#include "CommonType.h"

class CHttpFacade;// forward declare

class _IHttpFacadeEvent
{
public:
	virtual void OnHttpCompleted(CHttpFacade *pHttpF, bool bSuccess, bool bCancel) = 0;
	virtual void OnHttpProgress(CHttpFacade *pHttpF, DWORD dwTotalSize, DWORD dwCurrentSize) = 0;
	virtual void OnRequestTimeout(CHttpFacade *pHttpF, DWORD dwSeconds) = 0;
};


class CHttpFacadeHelperWnd : 
#if defined(WIN32)	// __KEH_USE_LKWND__
  public _ITinyWindowEvents
#else
  public IGTRunLoopObserver
#endif
{
public:
	CHttpFacadeHelperWnd();
	virtual ~CHttpFacadeHelperWnd();

	std::list<CHttpFacade*> m_listHttpFacade;

	void Initialize(void) { }
	void SetHttpFacadeTimer(CHttpFacade *, int nElapse);
	void KillHttpFacadeTimer(CHttpFacade *);
	void Finish(void) { }

#if defined(WIN32)	
	HWND GetHwnd(void)
	{
		return m_pWnd->GetHWND();
	}
#else
	IGTEventDelegate * GetDelegate(void)
	{
		return m_pEventDelegate;
	}	
#endif	

private:
	bool OnTimerProcess(WPARAM wParam, LPARAM lParam);
	bool OnSignalProcess(int nMsg, WPARAM wParam, LPARAM lParam);
	
protected:
#if defined(WIN32)		
	BOOL ITinyWindow_OnWndProc(ITinyWindow * pWindow, UINT uMsg, WPARAM wParam, LPARAM lParam);
#else
	virtual bool OnGTRunLoopObserver_SourceEvent(GT_EVENT_HANDLE handle, GT_PARAM param1, GT_PARAM param2);
	virtual bool OnGTRunLoopObserver_TimerEvent(GT_EVENT_HANDLE handle, GT_PARAM param);	
#endif	

protected:
	int m_nTimerID_Serial;

#if defined(WIN32)	
	ITinyWindow *m_pWnd;
#else	
	IGTEventDelegate *		m_pEventDelegate;
#endif	
};


/**
	@ingroup	NPhone	
	@brief	CHttpSession  Facade  Request ŸӾƿ ޽ ̿ safe  
	@note	
		CHttpFacade  ϴ Ŭ session().SetEvents()  ȣ .
		session().SendRequest()  ȣ  CHttpFacade::SendRequest()  ȣؾ ŸӾƿ .

	@author	taekwonv  [2 contact 2 me : taekwonv@nhncorp.com]
*/
class CHttpFacade :	protected _IHttpSessionEvents
{
public:
	CHttpFacade(void);
	virtual ~CHttpFacade(void);
	CHttpSession &session(){ return m_http; }
	bool SendRequest(_IHttpFacadeEvent * pResultEventReceiver, int nTimeoutElapse = 20000);	
	void StopRequest();

	std::string m_strNPhoneResultCode;
	bool IsWorking(void) { return m_bWorking; }
	void SetWorking(bool bWorking) { m_bWorking = bWorking; }
    bool GetProgressInfo(size_t &totalSize, size_t &currentSize);
    
protected:
	// _IHttpSessionEvents
	virtual void On_Http_SendProgress(IHttpSession * pSession, DWORD dwTotalSize, DWORD dwCurrentSize);
	virtual void On_Http_ReceiveProgress(IHttpSession * pSession, DWORD dwTotalSize, DWORD dwCurrentSize);
	virtual void On_Http_Completed(IHttpSession * pSession, bool bSuccess);

	CHttpSession		m_http;
	_IHttpFacadeEvent * m_pEvent;
	int m_nHttpTimeOut_TimerID;
	bool m_bStopped;
	bool m_bWorking;

private:
	friend class CHttpFacadeHelperWnd;
	static CHttpFacadeHelperWnd * ms_pAsyncEventRelayWnd;		// 踦 ѱ   ޽ 
	void CALLBACK OnTimeOut();
	UINT m_nHttpReqTimeoutElapse;

public:
	//CGTNormLog m_log;
	trid_t m_trid;
};


